package com.yaohun.farmingwar.effects;

import com.yaohun.farmingwar.FarmingWar;
import com.yaohun.farmingwar.game.Game;
import com.yaohun.farmingwar.game.GameSite;
import com.yaohun.farmingwar.game.Region;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Particle;
import org.bukkit.block.Block;
import org.bukkit.block.data.Ageable;
import org.bukkit.block.data.BlockData;
import org.bukkit.block.data.type.Farmland;
import org.bukkit.entity.Cow;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Player;
import org.bukkit.entity.Villager;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;
import org.bukkit.util.Vector;
import tools.CDTimeAPI;
import tools.RandomUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

public class SummonTheVillager {

    public static void apply(Game game, Player zhubo, String userName, int amount) {
        if (!game.isStarted()) {
            return;
        }
        // 获取主播的uuid
        UUID uuid = zhubo.getUniqueId();
        // 计时器key
        String timepieceKey = "SummonVillagerEffect";
        startTimepieceEvent(game,zhubo,userName,timepieceKey);
        if(CDTimeAPI.isCD(uuid,timepieceKey)){
            long newTime = CDTimeAPI.getCD(uuid,timepieceKey) + (245L * amount);
            CDTimeAPI.setPlayerCD(uuid,timepieceKey,newTime);
        } else {
            CDTimeAPI.setPlayerCD(uuid,timepieceKey,245L * amount);
        }
    }

    public static void startTimepieceEvent(Game game,Player zhubo,String userName,String timepieceKey) {
        if (CDTimeAPI.isCD(zhubo.getUniqueId(), timepieceKey)) {
            return;
        }
        BukkitTask task = new BukkitRunnable() {
            @Override
            public void run() {
                if (!CDTimeAPI.isCD(zhubo.getUniqueId(), timepieceKey)) {
                    cancel();
                    return;
                }
                spawnMobs(game,userName);
            }
        }.runTaskTimer(FarmingWar.inst(), 0L, 5L);
        game.addTasks(task);
    }

    public static void spawnMobs(Game game,String userName){
        Region region = game.getRegion();
        Location location = GameSite.getMinerWheat_MinAmount_Location();
        location.setYaw(-90);
        Villager entity = (Villager) location.getWorld().spawnEntity(location, EntityType.VILLAGER);
        entity.setMaxHealth(20);
        entity.setProfession(Villager.Profession.ARMORER);
        entity.setVillagerType(Villager.Type.PLAINS);
        entity.setCustomNameVisible(true);
        entity.setCustomName("§a" + userName);
        BukkitTask task = new BukkitRunnable() {
            private int i = 0;
            // 计算将在多少i后自动消失
            private double remove_x_i = (region.getMax().getX() - region.getMin().getX()) * 7;
            private List<Integer> integerList = new ArrayList<>();
            @Override
            public void run() {
                Location location = entity.getLocation().clone();
                if(i >= remove_x_i){
                    if(!entity.isDead()) {
                        entity.remove();
                    }
                    cancel();
                    return;
                }
                if (entity.isDead() || location.getX() >= region.getMax().getX()+0.5) {
                    entity.remove();
                    cancel();
                    return;
                }
                Vector vector0 = new Vector(1, 0, 0);
                vector0.normalize();
                vector0.multiply(0.15);
                entity.setRotation(-90, 0);
                entity.setVelocity(vector0);
                // 获取生物当前所在位置的方块
                Block block = location.getBlock();
                // 判断这个方块是否在游戏场地
                if(region.isInRegion(block.getLocation())) {
                    // 获取这个方块的ID代号
                    int hashCode = block.hashCode();
                    // 判断这个方块是否已录入系统  若已录入则跳过
                    if(!integerList.contains(hashCode)){
                        integerList.add(hashCode);
                        // 判断方块是否是小麦
                        if (block.getType() == Material.WHEAT) {
                            // 判断小麦成熟度
                            BlockData blockData = block.getBlockData();
                            if (blockData instanceof Ageable) {
                                Ageable ageable = (Ageable) blockData;
                                ageable.setAge(7);
                                block.setBlockData(ageable);
                            }
                        } else {
                            Block next_block = block.getRelative(0,-1,0);
                            if(next_block.getType() == Material.FARMLAND){
                                // 检索到泥土方块后判断方块是否是湿润状态
                                if (next_block.getBlockData() instanceof Farmland farmland) {
                                    // 如果没有打湿则跳过
                                    if (farmland.getMoisture() != 7) {
                                        farmland.setMoisture(7);
                                        next_block.setBlockData(farmland);
                                    }else {
                                        block.setType(Material.WHEAT);
                                        if (block.getBlockData() instanceof Ageable ageable) {
                                            ageable.setAge(3);
                                            block.setBlockData(ageable);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                if (RandomUtil.getRandomInt(1, 100) >= 50) {
                    block.getWorld().spawnParticle(Particle.VILLAGER_HAPPY, block.getLocation().add(0.5, 0.5, 0.5), 2, 0.5, 0.5, 0.5, 0.1);
                }
                i++;
            }
        }.runTaskTimer(FarmingWar.inst(), 1L, 1L);
        game.addTasks(task);
    }

}
