package com.yaohun.farmingwar.game;

import com.sk89q.worldedit.EditSession;
import com.sk89q.worldedit.WorldEdit;
import com.sk89q.worldedit.bukkit.BukkitAdapter;
import com.sk89q.worldedit.math.BlockVector3;
import com.sk89q.worldedit.regions.CuboidRegion;
import com.yaohun.farmingwar.FarmingWar;
import com.yaohun.farmingwar.data.LikeData;
import com.yaohun.farmingwar.manager.GameManager;
import org.bukkit.*;
import org.bukkit.block.Block;
import org.bukkit.boss.BarColor;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BossBar;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Firework;
import org.bukkit.entity.Item;
import org.bukkit.entity.Player;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.FireworkMeta;
import org.bukkit.plugin.Plugin;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;
import org.bukkit.util.Vector;
import tools.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;

public class Game {

    private Region region;
    private World world;
    private BossBar bossBar1;
    private BossBar bossBar2;
    private BossBar bossBar3;
    private boolean started = false;
    private boolean checking = false;
    private int completeAmount;
    private int completeGoal;
    private List<BukkitTask> tasks = new ArrayList<>();
    private List<Entity> entities = new ArrayList<>();
    public Game(GameManager manager){
        this.world = Bukkit.getWorld("world");
        this.region = new Region(world,new Point(-224,73,246),new Point(-204,73,266));
        bossBar1 = Bukkit.createBossBar("今日挑战进度", BarColor.WHITE, BarStyle.SEGMENTED_10);
        bossBar2 = Bukkit.createBossBar("补种进度", BarColor.GREEN, BarStyle.SEGMENTED_20);
        bossBar3 = Bukkit.createBossBar("点赞累积数量", BarColor.WHITE, BarStyle.SEGMENTED_10);
        this.completeGoal = manager.getGameGoal();
        this.countdownTime = manager.getGoalCountdown();
    }

    private void updateCompleteGoalBossBar() {
        double d = (double) completeAmount / (double) completeGoal;
        String bossTitle = "§6今日挑战进度: §f"+completeAmount+"/"+completeGoal;
        bossBar1.setTitle(bossTitle);
        BossBarUtil.setBarColor(bossBar1,d);
        BossBarUtil.setBarProgress(bossBar1,d);
    }

    private void updateCompleteWheatGoalBossBar() {
        int cNow = GameSite.getCompleteWheatAmount();
        int cMax = GameSite.getCompleteWheatGoal();
        double d = (double) cNow / (double) cMax;
        double d2 = MathUtil.round(d * 100, 2);
        String bossTitle = "§6小麦生长: §f"+cNow+"/"+cMax+" §b("+d2+"%)";
        bossBar2.setTitle(bossTitle);
        BossBarUtil.setBarColor(bossBar2,d);
        BossBarUtil.setBarProgress(bossBar2,d);
    }
    public void updateDianZanBossBar() {
        int dz = 0;
        int needDz = 300;
        String eventName = "";
        HashMap<String, LikeData> likeDataMap = FarmingWar.getGameManager().getLikeDataMap();
        for (String key : likeDataMap.keySet()){
            LikeData likeData = likeDataMap.get(key);
            dz = likeData.getNowAmount();
            needDz = likeData.getNeedAmount();
            eventName = likeData.getEffectKey();
            break;
        }
        int dz_need = needDz;
        double dz1 = (double) dz / (double) dz_need;
        String bossTitle = "§6"+eventName+"(戳一戳): §d"+dz+"§f/§c"+dz_need;
        bossBar3.setTitle(bossTitle);
        BossBarUtil.setBarColor(bossBar3,dz1);
        BossBarUtil.setBarProgress(bossBar3,dz1);
    }

    public void start() {
        if (started) {
            return;
        }
        this.started = true;
        this.checking = false;
        initWorld();
        GameUtil.loadSchematics("muban");
        this.completeAmount = 0;
        tasks.add(new BukkitRunnable() {
            @Override
            public void run() {
                updateCompleteGoalBossBar();
                updateCompleteWheatGoalBossBar();
                updateDianZanBossBar();
                if (!checking) {
                    checkSiteDegreeOfCompletion();
                }
            }
        }.runTaskTimer(FarmingWar.inst(), 0L, 10L));
    }
    public void reset() {
        // 删除当前游戏场地
        BlockVector3 vector1 = BlockVector3.at((region.getMin().getX()-1), 72, (region.getMin().getZ()-1));
        BlockVector3 vector2 = BlockVector3.at((region.getMax().getX()+1), 74,  (region.getMax().getZ()+1));
        CuboidRegion cuboidRegion = new CuboidRegion(BukkitAdapter.adapt(region.getWorld()), vector1, vector2);
        try (EditSession editSession = WorldEdit.getInstance().newEditSession(BukkitAdapter.adapt(region.getWorld()))) {
            editSession.setBlocks((com.sk89q.worldedit.regions.Region) cuboidRegion, BukkitAdapter.asBlockType(Material.AIR));
        } catch (Exception e) {
            e.printStackTrace();
        }
        // 重新生成游戏场地并重置游戏区域
        Bukkit.getScheduler().runTaskLater(FarmingWar.inst(), () -> {
            this.region = new Region(world,new Point(-224,73,246),new Point(-204,73,266));
        },1L);
        Bukkit.getScheduler().runTaskLater(FarmingWar.inst(), () -> {
            GameUtil.loadSchematics("muban");
        },5L);
        Bukkit.getScheduler().runTaskLater(FarmingWar.inst(), () -> {
            for (Player player : Bukkit.getOnlinePlayers()){
                player.teleport(region.getHub());
            }
        },10L);
    }
    public void initPlayerData(Player player) {
        player.setArrowCooldown(3600);
        player.setGameMode(GameMode.SURVIVAL);
        player.setAllowFlight(true);
        player.teleport(region.getHub());
        bossBar1.addPlayer(player);
        bossBar2.addPlayer(player);
        bossBar3.addPlayer(player);
        initPlayerBackpack(player);
    }
    public void initPlayerBackpack(Player player){
        Inventory inv = player.getInventory();
        inv.setItem(0, StackUtil.diamondSword());
        inv.setItem(1, StackUtil.trident());
        inv.setItem(2, StackUtil.quickPotion());
        inv.setItem(3, new ItemStack(Material.WHEAT_SEEDS,32));
        inv.setItem(4, new ItemStack(Material.BONE_MEAL,32));
    }
    private void initWorld() {
        this.world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
        this.world.setGameRule(GameRule.DO_WEATHER_CYCLE, false);
        this.world.setGameRule(GameRule.DO_MOB_SPAWNING, false);
        this.world.setGameRule(GameRule.DO_MOB_LOOT, false);
        this.world.setGameRule(GameRule.MOB_GRIEFING, false);
        this.world.setGameRule(GameRule.RANDOM_TICK_SPEED, 0);
        this.world.setTime(4500);
        this.world.setDifficulty(Difficulty.NORMAL);
    }

    private int countdown = 0;
    private int countdownTime = 11;
    public void checkSiteDegreeOfCompletion() {
        if(checking){return;}
        int i = GameSite.getCompleteWheatAmount();
        if(i >= GameSite.getCompleteWheatGoal()){
            checking = true;
            countdown = 0;
            BukkitTask task = new BukkitRunnable() {
                @Override
                public void run() {
                    int i = GameSite.getCompleteWheatAmount();
                    // 倒计时期间小麦受到破坏则停止倒计时
                    if(i < GameSite.getCompleteWheatGoal()){
                        tasks.remove(this);
                        checking = false;
                        Bukkit.broadcastMessage("§c[系统]§a小麦受到了破坏,需要进行修复.");
                        for (Player player : Bukkit.getOnlinePlayers()) {
                            player.sendTitle("§c哎呀!", "§6继续加油!");
                            player.playSound(player.getLocation(), "niganma", 1.0F, 1.0F);
                        }
                        cancel();
                        return;
                    }
                    if(countdown >= countdownTime-1){
                        tasks.add(new BukkitRunnable() {
                            private int i = 0;
                            @Override
                            public void run() {
                                if(i >= 20){
                                    cancel();
                                    return;
                                }
                                for (Player player : Bukkit.getOnlinePlayers()){
                                    shoutFirework(player.getLocation().clone().add(RandomUtil.getRandomInt(-10,10),RandomUtil.getRandomInt(3,6),RandomUtil.getRandomInt(-10,10)));
                                }
                                i++;
                            }
                        }.runTaskTimer(FarmingWar.inst(),0L,5L));
                        tasks.add(new BukkitRunnable() {
                            private int i = 0;
                            @Override
                            public void run() {
                                if (i >= 100) {
                                    cancel();
                                    return;
                                }
                                region.getWorld().setTime(region.getWorld().getTime() + 480L);
                                i++;
                            }
                        }.runTaskTimer(FarmingWar.inst(), 0L, 1L));
                        completeAmount++;
                        tasks.remove(this);
                        checking = false;
                        Bukkit.broadcastMessage("§c[系统]§a挑战成功! §b游戏进度+1");
                        GameUtil.sendAllTitle("§6§l收获成功","§f恭喜你完成挑战!",10,50,20);
                        for (Player player : Bukkit.getOnlinePlayers()) {
                            // 遍历玩家周围20x20x20的所有方块
                            // 获取玩家位置中心点的坐标
                            Location loc = player.getLocation();
                            int centerX = loc.getBlockX();
                            int centerY = loc.getBlockY();
                            int centerZ = loc.getBlockZ();
                            for (int x = centerX - 10; x <= centerX + 10; x++) {
                                for (int y = centerY - 10; y <= centerY + 10; y++) {
                                    for (int z = centerZ - 10; z <= centerZ + 10; z++) {
                                        // 获取方块
                                        Block block = player.getWorld().getBlockAt(x, y, z);
                                        // 示例操作：输出方块的类型
                                        if (block.getType() != Material.AIR && region.isInRegion(block.getLocation())) {
                                            block.breakNaturally();
                                        }
                                    }
                                }
                            }
                            // 获取玩家周围20x20x20区域内的所有实体
                            List<Entity> nearbyEntities = player.getWorld().getEntities();
                            for (Entity entity : nearbyEntities) {
                                if (entity instanceof Item) {  // 检查是否为 Item 实体
                                    Location itemLocation = entity.getLocation();
                                    // 判断实体是否在20x20x20范围内
                                    if (Math.abs(itemLocation.getX() - loc.getX()) <= 15 &&
                                            Math.abs(itemLocation.getY() - loc.getY()) <= 15 &&
                                            Math.abs(itemLocation.getZ() - loc.getZ()) <= 15) {
                                        entity.setGlowing(true);
                                        BukkitTask task = new BukkitRunnable() {
                                            private int i = 0;
                                            @Override
                                            public void run() {
                                                if(entity.isDead() || i >= 50){
                                                    if(!entity.isDead()){
                                                        entity.remove();
                                                    }
                                                    cancel();
                                                    return;
                                                }
                                                // 获取玩家当前位置和物品当前位置的方向向量
                                                Vector direction = loc.toVector().subtract(itemLocation.toVector()).normalize();
                                                // 设置物品实体的速度，让它朝玩家移动
                                                entity.setVelocity(direction.multiply(0.2)); // 调整速度大小
                                                i++;
                                            }
                                        }.runTaskTimer(FarmingWar.inst(),0,2L);
                                        addTasks(task);
                                    }
                                }
                            }
                            player.playSound(player.getLocation(), "duolaameng", 1.0F, 1.0F);
                            player.playSound(player.getLocation(), Sound.ENTITY_PLAYER_LEVELUP, 1.0f, 1.1f);
                        }
                        GameSite.restoreInitialization();
                        cancel();
                        return;
                    }
                    countdownEvent();
                }
            }.runTaskTimer(FarmingWar.inst(),0,20L);
            addTasks(task);
        }
    }
    private void countdownEvent(){
        countdown++;
        int daojishi = countdownTime;
        for (Player player : Bukkit.getOnlinePlayers()) {
            player.sendTitle("§6§l" + (daojishi - countdown), "§6即将挑战成功!");
            player.playSound(player.getLocation(), "daojishi", 1.0f, 1.0f);
        }
    }

    public Region getRegion() {return region;}
    public boolean isStarted() {return started;}
    public List<BukkitTask> getTasks() {
        return tasks;
    }
    public void addTasks(BukkitTask task){
        this.tasks.add(task);
    }

    private void shoutFirework(Location location){
        // 创建烟花对象
        Firework firework = location.getWorld().spawn(location.clone().add(0,1,0), Firework.class);
        FireworkMeta fireworkMeta = firework.getFireworkMeta();
        Color color = Color.YELLOW;
        int rand = RandomUtil.getRandomInt(1,100);
        if(rand >= 75){
            color = Color.RED;
        }else if(rand >= 50){
            color = Color.AQUA;
        }else if(rand >= 25){
            color = Color.GREEN;
        }
        // 设置烟花效果
        FireworkEffect fireworkEffect = FireworkEffect.builder()
                .withColor(color)
                .with(FireworkEffect.Type.BURST)
                .build();
        firework.setVelocity(new Vector(0, -0.2, 0));
        fireworkMeta.addEffect(fireworkEffect);
        fireworkMeta.setPower(0); // 设置烟花强度
        firework.setFireworkMeta(fireworkMeta);
        new BukkitRunnable() {
            public void run() {
                firework.detonate();
                cancel();
            }
        }.runTaskLater( FarmingWar.inst(), 5L );
    }

    public void effect_setComplete(int amount){
        completeAmount = amount;
    }
    public void effect_addComplete(int amount){
        effect_setComplete(completeAmount + amount);
    }
    public void effect_takeComplete(int amount){
        effect_setComplete(completeAmount - amount);
    }

    public BossBar getBossBar1() {
        return bossBar1;
    }

    public BossBar getBossBar2() {
        return bossBar2;
    }

    public void removeBossBar(Player player, BossBar bossBar) {
        bossBar.removePlayer(player);
    }
    public void addBossBar(Player player,BossBar bossBar) {
        bossBar.addPlayer(player);
    }

    public void createRegion(String regionKey, Region region) {
        FileConfiguration config = FarmingWar.inst().getConfig();
        ConfigurationSection section = config.createSection(regionKey);
        section.set("world", region.getWorld().getName());
        Point point1 = region.getMin();
        Point point2 = region.getMax();
        ConfigurationSection section1 = section.createSection("min");
        section1.set("x", point1.getX());
        section1.set("y", point1.getY());
        section1.set("z", point1.getZ());
        ConfigurationSection section2 = section.createSection("max");
        section2.set("x", point2.getX());
        section2.set("y", point2.getY());
        section2.set("z", point2.getZ());
        FarmingWar.inst().saveConfig();
    }
}
