package com.yaohun.farmingwar.listener;

import com.yaohun.farmingwar.FarmingWar;
import com.yaohun.farmingwar.game.Game;
import com.yaohun.farmingwar.game.Region;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.block.data.Ageable;
import org.bukkit.block.data.BlockData;
import org.bukkit.block.data.type.Farmland;
import org.bukkit.entity.*;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityExplodeEvent;
import org.bukkit.event.entity.ProjectileLaunchEvent;
import org.bukkit.event.player.PlayerSwapHandItemsEvent;
import org.bukkit.event.server.ServerListPingEvent;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.bukkit.scheduler.BukkitRunnable;
import tools.StackUtil;

import java.util.Iterator;

public class GameListener implements Listener {

    private FarmingWar farmingWar;
    private Game game;
    public GameListener(FarmingWar farmingWarMainClass){
        farmingWar = farmingWarMainClass;
        game = FarmingWar.getGame();
    }

    @EventHandler
    public void onPingMotd(ServerListPingEvent e){
        if(game.isStarted()){
            e.setMotd("§a整蛊模式: §6保卫小麦  §a当前版本: §6复刻版 §b运行中");
        } else {
            e.setMotd("§a整蛊模式: §6保卫小麦  §a当前版本: §6复刻版 §b运行中");
        }
    }

    /*主播按下F后将获得速度II的效果和急迫效果*/
    @EventHandler
    public void onSwap(PlayerSwapHandItemsEvent e) {
        Player p = e.getPlayer();
        e.setCancelled(true);
        if (!p.isSneaking()) {
            p.addPotionEffect(new PotionEffect(PotionEffectType.SPEED, 20 * 60 * 30, 1,false,false));
            p.teleport(game.getRegion().getHub());
        }
    }

    @EventHandler
    public void onDamageByEntity(EntityDamageByEntityEvent e){
        Entity entity = e.getEntity();
        if(entity instanceof Player player){
            if(e.getDamager() instanceof Bee) {
                ((Bee) e.getDamager()).damage(100);
                ((Player) entity).damage(1.0);
                if(player.isFlying()){
                    player.setFlying(false);
                }
            } else {
                e.setDamage(0);
                if(player.isFlying()){
                    player.setFlying(false);
                }
            }
        }
    }

    @EventHandler /*生物免摔落伤害*/
    public void onAntiFall(EntityDamageEvent event) {
        if (event.getCause().equals(EntityDamageEvent.DamageCause.FALL)) {
            event.setCancelled(true);
        }
    }

    /*
    * 爆炸会检测被炸的方块是否是小麦
    * 如果是小麦并有小麦的情况下会将小麦变成种子状态
    * */
    @EventHandler
    public void onEntityExplode(EntityExplodeEvent event) {
        if(game.isStarted()){
            Iterator<Block> iterator = event.blockList().iterator();
            while (iterator.hasNext()) {
                Block block = iterator.next();
                Region region = game.getRegion();
                if (region.isInRegion(block.getLocation())) {
                    if (block.getType() == Material.WHEAT) {
                        BlockData blockData = block.getBlockData();
                        if (blockData instanceof Ageable ageable) {
                            if (ageable.getAge() >= 3) {
                                block.setType(Material.AIR);
                            }
                        }
                    }
                    iterator.remove();
                } else if(region.isInRegionPlayer(block.getLocation())){
                    if(block.getType() == Material.FARMLAND){
                        Block wheatBlock = block.getRelative(0,1,0);
                        if (wheatBlock.getType() != Material.WHEAT) {
                            if (block.getBlockData() instanceof Farmland farmland) {
                                if (farmland.getMoisture() == farmland.getMaximumMoisture()) {
                                    block.setType(Material.FARMLAND);
                                }
                            }
                        }
                    }
                    iterator.remove();
                }
            }
        }
    }

    /*
    * 当玩家投掷三叉戟时会给玩家背包刷新一个三叉戟
    * */
    @EventHandler
    public void onTridentThrow(ProjectileLaunchEvent event) {
        if (event.getEntity() instanceof Trident) {
            Trident trident = (Trident) event.getEntity();
            if(trident.getShooter() instanceof Player player){
                player.getInventory().setItem(1, StackUtil.trident());
            }
            new BukkitRunnable() {
                @Override
                public void run() {
                    if (!trident.isDead()) {
                        trident.remove(); // 移除三叉戟
                    }
                }
            }.runTaskLater(FarmingWar.inst(), 5 * 20L); // 5秒延迟，1秒 = 20 tick
        }
        if (event.getEntity() instanceof ThrownPotion) {
            ThrownPotion potion = (ThrownPotion) event.getEntity();
            // 获取投掷者
            if (potion.getShooter() instanceof Player player) {
                event.setCancelled(true);
            }
        }
    }


    /*
     * 禁止破坏游戏区域外的方块
     * */
    @EventHandler
    public void onBlockBreak(BlockBreakEvent event) {
        Block block = event.getBlock();
        if (game.isStarted()) {
            Region region = game.getRegion();
            if (region.isInRegionPlayer(block.getLocation())) {
                event.setCancelled(true);
            }
        }
    }
}
